import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import classNames from 'classnames';
import PropTypes from 'prop-types';
import { forwardRef, useMemo } from 'react';
import styled, { useTheme } from 'styled-components';
import { FillLevelProvider, isFillLevel, toFillLevel, useFillLevel, } from './contexts/FillLevelContext';
import Button from './Button';
import ErrorIcon from './icons/ErrorIcon';
import InfoIcon from './icons/InfoIcon';
import StatusOkIcon from './icons/StatusOkIcon';
import WarningIcon from './icons/WarningIcon';
const SEVERITIES = ['info', 'danger', 'warning', 'success'];
const DEFAULT_SEVERITY = 'info';
const severityToIconColorKey = {
    info: 'icon-info',
    success: 'icon-success',
    warning: 'icon-warning',
    danger: 'icon-danger',
};
const severityToBorderColorKey = {
    info: 'border-info',
    success: 'border-success',
    warning: 'border-warning',
    danger: 'border-danger',
};
const severityToText = {
    info: 'Info',
    success: 'Success',
    warning: 'Warning',
    danger: 'Danger',
};
const severityToIcon = {
    info: InfoIcon,
    success: StatusOkIcon,
    warning: WarningIcon,
    danger: ErrorIcon,
};
const sizeToIconSize = {
    compact: 16,
    full: 20,
};
export function CalloutButton(props) {
    return (_jsx(Button, Object.assign({ secondary: true }, props)));
}
const Callout = forwardRef(({ title, severity = DEFAULT_SEVERITY, size = 'full', fillLevel, className, buttonProps, children, }, ref) => {
    severity = useMemo(() => {
        if (!severityToIconColorKey[severity]) {
            console.warn(`Callout: Incorrect severity (${severity}) specified. Valid values are ${SEVERITIES.map(s => `"${s}"`).join(', ')}. Defaulting to "${DEFAULT_SEVERITY}".`);
            return DEFAULT_SEVERITY;
        }
        return severity;
    }, [severity]);
    const theme = useTheme();
    const text = severityToText[severity];
    const iconColor = theme.colors[severityToIconColorKey[severity]];
    const borderColorKey = severityToBorderColorKey[severity];
    const Icon = severityToIcon[severity];
    const parentFillLevel = useFillLevel();
    fillLevel = toFillLevel(Math.max(2, isFillLevel(fillLevel) && fillLevel >= 0
        ? fillLevel
        : parentFillLevel + 1));
    let iconTopMargin = size === 'full' ? 0 : 2;
    if (title) {
        iconTopMargin += 2;
    }
    return (_jsx(FillLevelProvider, Object.assign({ value: fillLevel }, { children: _jsxs(CalloutWrap, Object.assign({ className: className, "$borderColorKey": borderColorKey, "$fillLevel": fillLevel, "$size": size, ref: ref }, { children: [_jsx("div", Object.assign({ className: "icon" }, { children: _jsx(Icon, { marginTop: iconTopMargin, size: sizeToIconSize[size], color: iconColor, display: "flex" }) })), _jsxs("div", { children: [_jsxs("h6", Object.assign({ className: classNames({ visuallyHidden: !title }) }, { children: [_jsx("span", Object.assign({ className: "visuallyHidden" }, { children: `${text}: ` })), title] })), _jsx("div", Object.assign({ className: "children" }, { children: children })), buttonProps && (_jsx("div", Object.assign({ className: "buttonArea" }, { children: _jsx(CalloutButton, Object.assign({}, buttonProps)) })))] })] })) })));
});
const CalloutWrap = styled.div(({ theme, $size, $fillLevel, $borderColorKey, }) => (Object.assign(Object.assign({ position: 'relative', display: 'flex', gap: theme.spacing.small, padding: $size === 'compact'
        ? `${theme.spacing.xsmall}px ${theme.spacing.medium}px`
        : `${theme.spacing.medium}px`, margin: 0, borderRadius: theme.borderRadiuses.medium }, theme.partials.text.body2LooseLineHeight), { backgroundColor: $fillLevel >= 3 ? theme.colors['fill-three'] : theme.colors['fill-two'], color: theme.colors['text-light'], h6: Object.assign(Object.assign({}, theme.partials.text.body1Bold), { color: theme.colors.text, margin: 0, padding: 0, marginBottom: theme.spacing.xxsmall }), '.children *:first-child': {
        marginTop: '0',
    }, '.children *:last-child': {
        marginBottom: '0',
    }, '.buttonArea': {
        display: 'flex',
        gap: theme.spacing.xsmall,
        marginTop: $size === 'compact' ? theme.spacing.xsmall : theme.spacing.medium,
    }, '&::before, &::after': {
        content: '""',
        position: 'absolute',
        left: 0,
        top: 0,
        bottom: 0,
        right: 0,
        pointerEvents: 'none',
    }, '&::before': {
        borderTopLeftRadius: theme.borderRadiuses.medium,
        borderBottomLeftRadius: theme.borderRadiuses.medium,
        right: 'unset',
        width: 3,
        background: theme.colors[$borderColorKey],
        zIndex: 2,
    }, '&::after': {
        borderRadius: theme.borderRadiuses.medium,
        border: $fillLevel >= 3 ? theme.borders['fill-three'] : theme.borders['fill-two'],
        content: '""',
        position: 'absolute',
        left: 0,
        top: 0,
        bottom: 0,
        right: 0,
        pointerEvents: 'none',
    }, '.visuallyHidden': {
        position: 'absolute',
        opacity: 0,
        width: 0,
        height: 0,
    }, '& a, & a:any-link': Object.assign({}, theme.partials.text.inlineLink) })));
Callout.propTypes = {
    severity: PropTypes.oneOf(SEVERITIES),
    title: PropTypes.string,
    size: PropTypes.oneOf(['compact', 'full']),
    fillLevel: PropTypes.oneOf([0, 1, 2, 3]),
    className: PropTypes.string,
};
export default Callout;
//# sourceMappingURL=Callout.js.map