import classNames from 'classnames'
import PropTypes from 'prop-types'
import { PropsWithChildren, forwardRef, useMemo } from 'react'
import styled, { useTheme } from 'styled-components'
import { ColorKey, Severity } from 'src/types'

import {
  FillLevel,
  FillLevelProvider,
  isFillLevel,
  toFillLevel,
  useFillLevel,
} from './contexts/FillLevelContext'
import Button, { ButtonProps } from './Button'
import ErrorIcon from './icons/ErrorIcon'
import InfoIcon from './icons/InfoIcon'
import StatusOkIcon from './icons/StatusOkIcon'
import WarningIcon from './icons/WarningIcon'

const SEVERITIES = ['info', 'danger', 'warning', 'success'] as const

export type CalloutSeverity = Extract<Severity, typeof SEVERITIES[number]>
const DEFAULT_SEVERITY: CalloutSeverity = 'info'

export type CalloutSize = 'compact' | 'full'

const severityToIconColorKey: Record<CalloutSeverity, ColorKey> = {
  info: 'icon-info',
  success: 'icon-success',
  warning: 'icon-warning',
  danger: 'icon-danger',
}

const severityToBorderColorKey: Record<CalloutSeverity, ColorKey> = {
  info: 'border-info',
  success: 'border-success',
  warning: 'border-warning',
  danger: 'border-danger',
}

const severityToText: Record<CalloutSeverity, string> = {
  info: 'Info',
  success: 'Success',
  warning: 'Warning',
  danger: 'Danger',
}

const severityToIcon: Record<CalloutSeverity, any> = {
  info: InfoIcon,
  success: StatusOkIcon,
  warning: WarningIcon,
  danger: ErrorIcon,
}

const sizeToIconSize: Record<CalloutSize, number> = {
  compact: 16,
  full: 20,
}

export type CalloutProps = PropsWithChildren<{
  title?: string
  severity?: CalloutSeverity
  size?: CalloutSize
  buttonProps?: ButtonProps
  fillLevel?: FillLevel
  className?: string
}>

export function CalloutButton(props: ButtonProps) {
  return (
    <Button
      secondary
      {...props}
    />
  )
}

const Callout = forwardRef<HTMLDivElement, CalloutProps>(({
  title,
  severity = DEFAULT_SEVERITY,
  size = 'full',
  fillLevel,
  className,
  buttonProps,
  children,
},
ref) => {
  severity = useMemo(() => {
    if (!severityToIconColorKey[severity]) {
      console.warn(`Callout: Incorrect severity (${severity}) specified. Valid values are ${SEVERITIES.map(s => `"${s}"`).join(', ')}. Defaulting to "${DEFAULT_SEVERITY}".`)

      return DEFAULT_SEVERITY
    }

    return severity
  }, [severity])
  const theme = useTheme()

  const text = severityToText[severity]
  const iconColor = theme.colors[severityToIconColorKey[severity]]
  const borderColorKey = severityToBorderColorKey[severity]
  const Icon = severityToIcon[severity]
  const parentFillLevel = useFillLevel()

  fillLevel = toFillLevel(Math.max(2,
    isFillLevel(fillLevel) && fillLevel >= 0
      ? fillLevel
      : parentFillLevel + 1))

  let iconTopMargin = size === 'full' ? 0 : 2

  if (title) {
    iconTopMargin += 2
  }

  return (
    <FillLevelProvider value={fillLevel}>
      <CalloutWrap
        className={className}
        $borderColorKey={borderColorKey}
        $fillLevel={fillLevel}
        $size={size}
        ref={ref}
      >
        <div className="icon">
          <Icon
            marginTop={iconTopMargin}
            size={sizeToIconSize[size]}
            color={iconColor}
            display="flex"
          />
        </div>
        <div>
          <h6 className={classNames({ visuallyHidden: !title })}>
            <span className="visuallyHidden">{`${text}: `}</span>
            {title}
          </h6>
          <div className="children">{children}</div>
          {buttonProps && (
            <div className="buttonArea">
              <CalloutButton {...buttonProps} />
            </div>
          )}
        </div>
      </CalloutWrap>
    </FillLevelProvider>
  )
})

const CalloutWrap = styled.div<{
  $borderColorKey: string
  $size: CalloutSize
  $fillLevel: FillLevel
}>(({
  theme, $size, $fillLevel, $borderColorKey,
}) => ({
  position: 'relative',
  display: 'flex',
  gap: theme.spacing.small,
  padding:
    $size === 'compact'
      ? `${theme.spacing.xsmall}px ${theme.spacing.medium}px`
      : `${theme.spacing.medium}px`,
  margin: 0,
  borderRadius: theme.borderRadiuses.medium,
  ...theme.partials.text.body2LooseLineHeight,
  backgroundColor:
    $fillLevel >= 3 ? theme.colors['fill-three'] : theme.colors['fill-two'],
  color: theme.colors['text-light'],
  h6: {
    ...theme.partials.text.body1Bold,
    color: theme.colors.text,
    margin: 0,
    padding: 0,
    marginBottom: theme.spacing.xxsmall,
  },
  '.children *:first-child': {
    marginTop: '0',
  },
  '.children *:last-child': {
    marginBottom: '0',
  },
  '.buttonArea': {
    display: 'flex',
    gap: theme.spacing.xsmall,
    marginTop:
      $size === 'compact' ? theme.spacing.xsmall : theme.spacing.medium,
  },
  '&::before, &::after': {
    content: '""',
    position: 'absolute',
    left: 0,
    top: 0,
    bottom: 0,
    right: 0,
    pointerEvents: 'none',
  },
  '&::before': {
    borderTopLeftRadius: theme.borderRadiuses.medium,
    borderBottomLeftRadius: theme.borderRadiuses.medium,
    right: 'unset',
    width: 3,
    background: theme.colors[$borderColorKey],
    zIndex: 2,
  },
  '&::after': {
    borderRadius: theme.borderRadiuses.medium,
    border:
      $fillLevel >= 3 ? theme.borders['fill-three'] : theme.borders['fill-two'],
    content: '""',
    position: 'absolute',
    left: 0,
    top: 0,
    bottom: 0,
    right: 0,
    pointerEvents: 'none',
  },
  '.visuallyHidden': {
    position: 'absolute',
    opacity: 0,
    width: 0,
    height: 0,
  },
  '& a, & a:any-link': {
    ...theme.partials.text.inlineLink,
  },
}))

Callout.propTypes = {
  severity: PropTypes.oneOf(SEVERITIES),
  title: PropTypes.string,
  size: PropTypes.oneOf(['compact', 'full']),
  fillLevel: PropTypes.oneOf([0, 1, 2, 3]),
  className: PropTypes.string,
}

export default Callout
